@@ -53,6 +53,18 @@ template <typename num1, typename num2, typename num3> constexpr bool inRangeExc
  *   invocations of getDatePart().
  */
 
+/*!
+ * \struct DateTimeExpression
+ * \brief The DateTimeExpression struct holds information about a time expression (e.g. an ISO-8601 timestamp).
+ *
+ * This struct holds the date and time specification (DateTimeExpression::value), the time-zone delta
+ * (DateTimeExpression::delta) and what parts are actually specified (DateTimeExpression::parts).
+ *
+ * The idea is to be able to preserve a time-zone delta and present parts when deserializing and then
+ * serializing again. Note that it still does not allow to preserve the format of the original timestamp
+ * exactly.
+ */
+
 /*!
  * \brief Constructs a new DateTime object with the local time from the specified UNIX \a timeStamp.
  */
@@ -78,39 +90,14 @@ DateTime DateTime::fromTimeStamp(time_t timeStamp)
  */
 DateTime DateTime::fromString(const char *str)
 {
-    int values[7] = { 0 };
-    int *const dayIndex = values + 2;
-    int *const secondsIndex = values + 5;
-    int *valueIndex = values;
-    int *const valuesEnd = values + 7;
-    double millisecondsFact = 100.0, milliseconds = 0.0;
-    for (const char *strIndex = str;; ++strIndex) {
-        const char c = *strIndex;
-        if (c <= '9' && c >= '0') {
-            if (valueIndex > secondsIndex) {
-                milliseconds += (c - '0') * millisecondsFact;
-                millisecondsFact /= 10;
-            } else {
-                Detail::raiseAndAdd(*valueIndex, 10, c);
-            }
-        } else if ((c == '-' || c == ':' || c == '/') || (c == '.' && (valueIndex == secondsIndex))
-            || ((c == ' ' || c == 'T') && (valueIndex == dayIndex))) {
-            if (++valueIndex == valuesEnd) {
-                break; // just ignore further values for now
-            }
-        } else if (c == '\0') {
-            break;
-        } else {
-            throw ConversionException(argsToString("Unexpected character \"", c, '\"'));
-        }
-    }
-    return DateTime::fromDateAndTime(values[0], values[1], *dayIndex, values[3], values[4], *secondsIndex, milliseconds);
+    return DateTimeExpression::fromString(str).value;
 }
 
 /*!
  * \brief Parses the specified ISO date time denotation provided as C-style string.
  * \returns Returns a pair where the first value is the parsed date time and the second value
  *          the time zone designator (a time span which can be subtracted from the first value to get the UTC time).
+ * \throws Throws a ConversionException if the specified \a str does not match the expected time format.
  * \remarks
  * - Parsing durations and time intervals is *not* supported.
  * - Truncated representations are *not* supported.
@@ -119,90 +106,8 @@ DateTime DateTime::fromString(const char *str)
  */
 std::pair<DateTime, TimeSpan> DateTime::fromIsoString(const char *str)
 {
-    int values[9] = { 0 };
-    int *const yearIndex = values + 0;
-    int *const monthIndex = values + 1;
-    int *const dayIndex = values + 2;
-    int *const hourIndex = values + 3;
-    int *const secondsIndex = values + 5;
-    int *const miliSecondsIndex = values + 6;
-    int *const deltaHourIndex = values + 7;
-    int *const valuesEnd = values + 9;
-    int *valueIndex = values;
-    unsigned int remainingDigits = 4;
-    bool deltaNegative = false;
-    double millisecondsFact = 100.0, milliseconds = 0.0;
-    for (const char *strIndex = str;; ++strIndex) {
-        const char c = *strIndex;
-        if (c <= '9' && c >= '0') {
-            if (valueIndex == miliSecondsIndex) {
-                milliseconds += (c - '0') * millisecondsFact;
-                millisecondsFact /= 10;
-            } else {
-                if (!remainingDigits) {
-                    if (++valueIndex == miliSecondsIndex || valueIndex >= valuesEnd) {
-                        throw ConversionException("Max. number of digits exceeded");
-                    }
-                    remainingDigits = 2;
-                }
-                *valueIndex *= 10;
-                *valueIndex += c - '0';
-                remainingDigits -= 1;
-            }
-        } else if (c == 'T') {
-            if (++valueIndex != hourIndex) {
-                throw ConversionException("\"T\" expected before hour");
-            }
-            remainingDigits = 2;
-        } else if (c == '-') {
-            if (valueIndex < dayIndex) {
-                ++valueIndex;
-            } else if (++valueIndex >= secondsIndex) {
-                valueIndex = deltaHourIndex;
-                deltaNegative = true;
-            } else {
-                throw ConversionException("Unexpected \"-\" after day");
-            }
-            remainingDigits = 2;
-        } else if (c == '.') {
-            if (valueIndex != secondsIndex) {
-                throw ConversionException("Unexpected \".\"");
-            } else {
-                ++valueIndex;
-            }
-        } else if (c == ':') {
-            if (valueIndex < hourIndex) {
-                throw ConversionException("Unexpected \":\" before hour");
-            } else if (valueIndex == secondsIndex) {
-                throw ConversionException("Unexpected \":\" after second");
-            } else {
-                ++valueIndex;
-            }
-            remainingDigits = 2;
-        } else if ((c == '+') && (++valueIndex >= secondsIndex)) {
-            valueIndex = deltaHourIndex;
-            deltaNegative = false;
-            remainingDigits = 2;
-        } else if ((c == 'Z') && (++valueIndex >= secondsIndex)) {
-            valueIndex = deltaHourIndex + 2;
-            remainingDigits = 2;
-        } else if (c == '\0') {
-            break;
-        } else {
-            throw ConversionException(argsToString("Unexpected \"", c, '\"'));
-        }
-    }
-    auto delta = TimeSpan::fromMinutes(*deltaHourIndex * 60 + values[8]);
-    if (deltaNegative) {
-        delta = TimeSpan(-delta.totalTicks());
-    }
-    if (valueIndex < monthIndex && !*monthIndex) {
-        *monthIndex = 1;
-    }
-    if (valueIndex < dayIndex && !*dayIndex) {
-        *dayIndex = 1;
-    }
-    return make_pair(DateTime::fromDateAndTime(*yearIndex, *monthIndex, *dayIndex, *hourIndex, values[4], *secondsIndex, milliseconds), delta);
+    const auto expr = DateTimeExpression::fromIsoString(str);
+    return std::make_pair(expr.value, expr.delta);
 }
 
 /*!
@@ -466,4 +371,235 @@ int DateTime::getDatePart(DatePart part) const
     return 0;
 }
 
+/// \cond
+static DateTimeParts dateTimePartsFromParsingDistance(const int *valueIndex, const int *values)
+{
+    return static_cast<DateTimeParts>((1 << (valueIndex - values + 1)) - 1);
+}
+/// \endcond
+
+/*!
+ * \brief Parses the specified ISO date time denotation provided as C-style string.
+ * \throws Throws a ConversionException if the specified \a str does not match the expected time format.
+ * \remarks
+ * - Parsing durations and time intervals is *not* supported.
+ * - Truncated representations are *not* supported.
+ * - Standardised extensions (ISO 8601-2:2019) are *not* supported.
+ * \sa https://en.wikipedia.org/wiki/ISO_8601
+ */
+DateTimeExpression DateTimeExpression::fromIsoString(const char *str)
+{
+    auto res = DateTimeExpression();
+    int values[9] = { 0 };
+    int *const yearIndex = values + 0;
+    int *const monthIndex = values + 1;
+    int *const dayIndex = values + 2;
+    int *const hourIndex = values + 3;
+    int *const secondsIndex = values + 5;
+    int *const miliSecondsIndex = values + 6;
+    int *const deltaHourIndex = values + 7;
+    int *const valuesEnd = values + 9;
+    int *valueIndex = values;
+    unsigned int remainingDigits = 4;
+    bool deltaNegative = false;
+    double millisecondsFact = 100.0, milliseconds = 0.0;
+    for (const char *strIndex = str;; ++strIndex) {
+        const char c = *strIndex;
+        if (c <= '9' && c >= '0') {
+            if (valueIndex == miliSecondsIndex) {
+                milliseconds += (c - '0') * millisecondsFact;
+                millisecondsFact /= 10;
+            } else {
+                if (!remainingDigits) {
+                    if (++valueIndex == miliSecondsIndex || valueIndex >= valuesEnd) {
+                        throw ConversionException("Max. number of digits exceeded");
+                    }
+                    remainingDigits = 2;
+                }
+                *valueIndex *= 10;
+                *valueIndex += c - '0';
+                remainingDigits -= 1;
+            }
+        } else if (c == 'T') {
+            if (++valueIndex != hourIndex) {
+                throw ConversionException("\"T\" expected before hour");
+            }
+            remainingDigits = 2;
+        } else if (c == '-') {
+            if (valueIndex < dayIndex) {
+                ++valueIndex;
+            } else if (++valueIndex >= secondsIndex) {
+                valueIndex = deltaHourIndex;
+                deltaNegative = true;
+            } else {
+                throw ConversionException("Unexpected \"-\" after day");
+            }
+            remainingDigits = 2;
+        } else if (c == '.') {
+            if (valueIndex != secondsIndex) {
+                throw ConversionException("Unexpected \".\"");
+            } else {
+                ++valueIndex;
+            }
+        } else if (c == ':') {
+            if (valueIndex < hourIndex) {
+                throw ConversionException("Unexpected \":\" before hour");
+            } else if (valueIndex == secondsIndex) {
+                throw ConversionException("Unexpected \":\" after second");
+            } else {
+                ++valueIndex;
+            }
+            remainingDigits = 2;
+        } else if ((c == '+') && (++valueIndex >= secondsIndex)) {
+            valueIndex = deltaHourIndex;
+            deltaNegative = false;
+            remainingDigits = 2;
+        } else if ((c == 'Z') && (++valueIndex >= secondsIndex)) {
+            valueIndex = deltaHourIndex + 2;
+            remainingDigits = 2;
+        } else if (c == '\0') {
+            break;
+        } else {
+            throw ConversionException(argsToString("Unexpected \"", c, '\"'));
+        }
+    }
+    res.delta = TimeSpan::fromMinutes(*deltaHourIndex * 60 + values[8]);
+    if (deltaNegative) {
+        res.delta = TimeSpan(-res.delta.totalTicks());
+    }
+    if (valueIndex < monthIndex && !*monthIndex) {
+        *monthIndex = 1;
+    }
+    if (valueIndex < dayIndex && !*dayIndex) {
+        *dayIndex = 1;
+    }
+    res.value = DateTime::fromDateAndTime(*yearIndex, *monthIndex, *dayIndex, *hourIndex, values[4], *secondsIndex, milliseconds);
+    res.parts = dateTimePartsFromParsingDistance(valueIndex, values);
+    return res;
+}
+
+/*!
+ * \brief Parses the given C-style string.
+ * \throws Throws a ConversionException if the specified \a str does not match the expected time format.
+ *
+ * The expected format is something like "2012-02-29 15:34:20.033" or "2012/02/29 15:34:20.033". The
+ * delimiters '-', ':' and '/' are exchangeable.
+ *
+ * \sa DateTimeExpression::fromIsoString()
+ */
+DateTimeExpression DateTimeExpression::fromString(const char *str)
+{
+    auto res = DateTimeExpression();
+    int values[7] = { 0 };
+    int *const monthIndex = values + 1;
+    int *const dayIndex = values + 2;
+    int *const secondsIndex = values + 5;
+    int *valueIndex = values;
+    int *const valuesEnd = values + 7;
+    double millisecondsFact = 100.0, milliseconds = 0.0;
+    for (const char *strIndex = str;; ++strIndex) {
+        const char c = *strIndex;
+        if (c <= '9' && c >= '0') {
+            if (valueIndex > secondsIndex) {
+                milliseconds += (c - '0') * millisecondsFact;
+                millisecondsFact /= 10;
+            } else {
+                Detail::raiseAndAdd(*valueIndex, 10, c);
+            }
+        } else if ((c == '-' || c == ':' || c == '/') || (c == '.' && (valueIndex == secondsIndex))
+            || ((c == ' ' || c == 'T') && (valueIndex == dayIndex))) {
+            if (++valueIndex == valuesEnd) {
+                break; // just ignore further values for now
+            }
+        } else if (c == '\0') {
+            break;
+        } else {
+            throw ConversionException(argsToString("Unexpected character \"", c, '\"'));
+        }
+    }
+    if (valueIndex < monthIndex && !*monthIndex) {
+        *monthIndex = 1;
+    }
+    if (valueIndex < dayIndex && !*dayIndex) {
+        *dayIndex = 1;
+    }
+    res.value = DateTime::fromDateAndTime(values[0], values[1], *dayIndex, values[3], values[4], *secondsIndex, milliseconds);
+    res.parts = dateTimePartsFromParsingDistance(valueIndex, values);
+    return res;
+}
+
+/*!
+ * \brief Returns the string representation of the current instance in the ISO format.
+ * \remarks Only present parts will be included.
+ */
+std::string DateTimeExpression::toIsoString(char dateDelimiter, char timeDelimiter, char timeZoneDelimiter) const
+{
+    auto s = std::stringstream(std::stringstream::in | std::stringstream::out);
+    s << setfill('0');
+    if (parts & DateTimeParts::Year) {
+        s << setw(4) << value.year();
+    }
+    if (parts & DateTimeParts::Month) {
+        if (s.tellp()) {
+            s << dateDelimiter;
+        }
+        s << setw(2) << value.month();
+    }
+    if (parts & DateTimeParts::Day) {
+        if (s.tellp()) {
+            s << dateDelimiter;
+        }
+        s << setw(2) << value.day();
+    }
+    if (parts & DateTimeParts::Hour) {
+        if (s.tellp()) {
+            s << 'T';
+        }
+        s << setw(2) << value.hour();
+    }
+    if (parts & DateTimeParts::Minute) {
+        if (s.tellp()) {
+            s << timeDelimiter;
+        }
+        s << setw(2) << value.minute();
+    }
+    if (parts & DateTimeParts::Second) {
+        if (s.tellp()) {
+            s << timeDelimiter;
+        }
+        s << setw(2) << value.second();
+    }
+    if (parts & DateTimeParts::SubSecond) {
+        const auto milli = value.millisecond();
+        const auto micro = value.microsecond();
+        const auto nano = value.nanosecond();
+        s << '.' << setw(3) << milli;
+        if (micro || nano) {
+            s << setw(3) << micro;
+            if (nano) {
+                s << nano / TimeSpan::nanosecondsPerTick;
+            }
+        }
+    }
+    if (parts & DateTimeParts::TimeZoneDelta) {
+        auto d = delta;
+        if (d.isNegative()) {
+            s << '-';
+            d = TimeSpan(-d.totalTicks());
+        } else {
+            s << '+';
+        }
+        if (parts & DateTimeParts::DeltaHour) {
+            s << setw(2) << d.hours();
+        }
+        if (parts & DateTimeParts::DeltaMinute) {
+            if (parts & DateTimeParts::DeltaHour) {
+                s << timeZoneDelimiter;
+            }
+            s << setw(2) << d.minutes();
+        }
+    }
+    return s.str();
+}
+
 } // namespace CppUtilities
